/*   
 *   $Id: currency.c,v 1.11 2001/07/14 17:58:44 ondra Exp $
 * 
 *   Holiday Debts manager for PalmOS.
 *   Copyright (C) 2000,2001 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *   You can contact me at 'ondrap@penguin.cz'.
*/

#include <PalmOS.h>

#include "debtsRsc.h"
#include "debts.h"
#include "currency.h"
#include "txtask.h"
#include "db.h"
#include "float.h"
#include "main.h"

currencyListType *currencyList = NULL;
static Int16 selectedLine = -1;

static void
cur_draw_item(Int16 itemnum,RectangleType *bounds,
		 Char **itemtext)
{
    currencyType *record;
    char text[25];
    Int16 drawx;

    record = (currencyType *) itemtext[itemnum];

    WinDrawChars(record->name,StrLen(record->name),
		 bounds->topLeft.x,bounds->topLeft.y);

    if (dbprefs.indirectcur) {
	float_to_str(text,1.00/record->rate,4);
	StrPrintF(text+StrLen(text),"%s=1%s",record->name,dbprefs.lcurrency);
    } else {
	StrPrintF(text,"1%s=",record->name);
	float_to_str(text+StrLen(text),record->rate,4);
	StrCat(text,dbprefs.lcurrency);
    }

    drawx = bounds->topLeft.x + bounds->extent.x -
	15 - FntCharsWidth(text,StrLen(text));
    WinDrawChars(text,StrLen(text),
		 drawx,bounds->topLeft.y);
}


void
cur_destroy_list(void)
{
    Int16 i;

    if (currencyList != NULL) {
	for (i=0;i<currencyList->nitems;i++)
	    MemPtrFree(currencyList->list[i]);
	MemPtrFree(currencyList->list);
	MemPtrFree(currencyList);
	currencyList = NULL;
    }
}

void
cur_init_list()
{
    cur_destroy_list();
    currencyList = db_get_currency_list(dbprefs.lcurrency);
}

void
cur_init(void)
{
    LstSetListChoices(GetObjectPtr(lstCurrency),
		      (void *)(currencyList->list + 1),
		      currencyList->nitems - 1);
    LstSetDrawFunction(GetObjectPtr(lstCurrency),cur_draw_item);
}

Boolean
cur_change(Int16 curidx)
{
    Int16 id,i;
    char name[MAX_CURRENCY_NAME+1];
    Boolean selected;
    double rate;

    if (curidx == -1) {
	StrCopy(name,"");
	rate = 1.0;
    } else {
	StrCopy(name,currencyList->list[curidx]->name);
	rate = currencyList->list[curidx]->rate;
    }

    selected = txtask_dialog_rate(name,MAX_CURRENCY_NAME,&rate);

    if (!selected)
	return false;

    if (curidx == -1) {
	for (id=1,i=0;i<currencyList->nitems;i++) 
	    if (currencyList->list[i]->id >= id)
		id = currencyList->list[i]->id + 1;
    }
    else {
	id = currencyList->list[curidx]->id;
    }

    db_add_currency(id,name,rate);
    return true;
}

Boolean 
CurrencyFormHandleEvent (EventPtr e)
{
    Boolean handled = false;
    FormPtr frm;
    Int16 i;
    
    switch (e->eType) {
    case frmOpenEvent:
	selectedLine = -1;
	frm = FrmGetActiveForm();
	
	cur_init();

	FrmDrawForm(frm);
	handled = true;
	break;
	
    case frmCloseEvent:
	handled = false;
	break;

    case lstSelectEvent:
	i = e->data.lstSelect.selection;
	if (i == selectedLine) {
	    if (cur_change(i+1)) 
		FrmUpdateForm(frmPeople,updCurrencyList);
	} else
	    selectedLine = i;
	handled = true;
	break;

    case ctlSelectEvent:
	switch(e->data.ctlSelect.controlID) {
	case btnCurrencyDelete:
	    i = LstGetSelection(GetObjectPtr(lstCurrency));
	    if (i != noListSelection) {
		i++;
		if (db_currency_inuse(currencyList->list[i]->id)) 
		    FrmAlert(altCurrencyInuse);
		else if (FrmCustomAlert(altDeleteCurrencyOK,
					currencyList->list[i]->name,
					NULL,NULL) == 0) {
		    db_delete_currency(currencyList->list[i]->id);
		    FrmUpdateForm(frmCurrency,updCurrencyList);
		}
	    }
	    break;
	case btnCurrencyNew:
	    if (cur_change(-1)) 
		FrmUpdateForm(frmCurrency,updCurrencyList);
	    handled = true;
	    break;
	case btnCurrencyModify:
	    i = LstGetSelection(GetObjectPtr(lstCurrency));
	    if (i != noListSelection) {
		if (cur_change(i+1)) 
		    FrmUpdateForm(frmPeople,updCurrencyList);
	    }
	    handled = true;
	    break;
	case btnCurrencyDone:
	    FrmReturnToForm(0);
	    FrmUpdateForm(frmMain,updCurrencyList);
	    handled = true;
	    break;
	}
	break;
    case frmUpdateEvent:
	if (e->data.frmUpdate.updateCode == updCurrencyList) {
	    selectedLine = -1;
	    cur_init_list();
	    cur_init();
	    LstDrawList(GetObjectPtr(lstCurrency));
	    handled = true;
	}
    default:
        break;
    }

    return handled;
}
