/*   
 *   $Id: db.h,v 1.27 2002/07/12 19:30:32 ondra Exp $
 * 
 *   Holiday Debts manager for PalmOS.
 *   Copyright (C) 2000,2001 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *   You can contact me at 'ondrap@penguin.cz'.
*/

#ifndef _DB_H_
#define _DB_H_

#include "debtsRsc.h"

#define CARDNO 0
#define DBTYPE 'Data'
#define DBVERSION 5

#define MAX_CURRENCY_NAME 3

typedef struct {
    Int16 id;
    double coef;
}pmtType;

typedef struct {
    Int16 nitems;
    Int16 seq[0];
}dbItemList;

typedef struct {
    Int16 nitems;
    pmtType *people;
    char *names[0];
}dbPeopleList;

typedef enum {
    db_person=1,
    db_description=2,
    db_currency=3,
    db_item=5
}rectype;

typedef struct {
    Int16 id;
    double coef;
    char name[0];
}personType;

typedef struct {
    char name[MAX_CURRENCY_NAME+1];
    Int16 id;
    double rate;
}currencyType;

typedef struct {
    Int16 nitems;
    currencyType **list;
}currencyListType;

typedef struct {
    DateType date;
    double amount;
    Int16 payer;
    Boolean transfer;
    Int16 currency; /* Reserved for future use */
    char description[MAX_DESCRIPTION+1];
    Int16 npayees;
    pmtType *pmts;
}itemRec;

typedef struct {
    DateType date;
    double amount;
    Int16 payer;
    Boolean transfer;
    Int16 currency; /* Reserved for future use */
    char description[MAX_DESCRIPTION+1];
    Int16 npayees;
    Int16 unused;
    /* THIS MUST BE LAST ITEM IN ARRAY */
    pmtType pmts[0];
}itemType;

typedef struct {
    rectype type;
    union {
	personType person;
	itemType item;
	currencyType currency;
	char description[MAX_DESCRIPTION+1];
    }u;
}dbRecord;

typedef struct {
    Int16 curidx;
    Int16 fuser;
    Int16 firstline;
    char lcurrency[MAX_CURRENCY_NAME+1];
    Boolean indirectcur;
    Int16 costMode; /* Should be CostMode but we get nasty #include problms */
}dbPrefs;

typedef struct {
    Int16 curidx;
    Int16 fuser;
    Int16 firstline;
    char lcurrency[MAX_CURRENCY_NAME+1];
    Boolean indirectcur;
}dbPrefsV4;

typedef struct {
    Int16 curidx;
    Int16 fuser;
    Int16 firstline;
    char lcurrency[MAX_CURRENCY_NAME];
}dbPrefsV3;

Err db_close();
Boolean db_create_new(char *name,LocalID *id,UInt16 *cardno);
Boolean db_open_by_id(LocalID dbid,UInt16 cardno);
void db_free_peoplelist(dbPeopleList *list);
dbPeopleList * db_get_peoplelist(void);
void db_add_person(Int16 id,char *name,double coef);
Boolean db_delete_person(Int16 id);
Boolean db_delete_item(Int16 pos);
Boolean db_add_item(itemRec *item);
itemType * db_get_item(Int16 rnum);
void db_free_item(itemType *item);
UInt16 db_num_items(void);
double db_compute_balance(Int16 user,Int16 curidx);
double db_compute_cost(Int16 user,Int16 curidx);
dbItemList * db_get_itemlist(Int16 user);
void db_free_itemlist(dbItemList *arr);
void db_copy_template(LocalID dstid,UInt16 dstcardno,
		      LocalID srcid,UInt16 srccardno);
Err db_delete_database(LocalID id,UInt16 cardno);
void db_get_name(char *name);
void db_add_currency(Int16 id,char *name, double rate);
currencyListType * db_get_currency_list(char *lcurrency);
dbPrefs db_get_prefs(void);
void db_save_prefs(dbPrefs prefs);
Boolean db_currency_inuse(Int16 id);
Boolean db_delete_currency(Int16 id);
Boolean db_check_database(LocalID dbid,UInt16 cardno);
void db_add_description(char *description);
char ** db_get_descriptions(void);
void db_free_descriptions(char **descriptions);
Boolean db_delete_description(Int16 seq);
double db_compute_payment(Int16 user,Int16 curidx);
#endif
