/*   
 *   $Id: description.c,v 1.2 2002/03/09 13:02:10 ondra Exp $
 * 
 *   Holiday Debts manager for PalmOS.
 *   Copyright (C) 2000,2001 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *   You can contact me at 'ondrap@penguin.cz'.
*/

#include <PalmOS.h>

#include "debtsRsc.h"
#include "debts.h"
#include "db.h"
#include "txtask.h"

static char **descriptions = NULL;

static void
description_destroy()
{
    db_free_descriptions(descriptions);
    descriptions = NULL;
}

static void
description_init()
{
    Int16 i;

    if (descriptions != NULL)
	description_destroy();
    descriptions = db_get_descriptions();
    for (i=0;descriptions[i];i++)
	;
    LstSetListChoices(GetObjectPtr(lstDescription),descriptions,i);
}

static void
description_new()
{
    char text[MAX_DESCRIPTION+1];
    
    StrCopy(text,"");
    if (txtask_dialog(strNewDescr,strDescr,text,MAX_DESCRIPTION)) {
	db_add_description(text);
	FrmUpdateForm(frmDescription,updDescriptionList);
    }
}

Boolean
DescriptionFormHandleEvent (EventPtr e)
{
    Boolean handled = false;
    FormPtr frm;
    Int16 i;
    
    switch (e->eType) {
    case frmOpenEvent:
	frm = FrmGetActiveForm();
	description_init();
	FrmDrawForm(frm);
	handled = true;
	break;
	
    case frmCloseEvent:
	description_destroy();
	handled = false;
	break;

    case ctlSelectEvent:
	switch(e->data.ctlSelect.controlID) {
	case btnDescriptionNew:
	    description_new();
	    handled = true;
	    break;
	case btnDescriptionDone:
	    description_destroy();
	    FrmReturnToForm(0);
	    handled = true;
	    break;
	case btnDescriptionDelete:
	    i = LstGetSelection(GetObjectPtr(lstDescription));
	    if (i != noListSelection) 
		if (FrmAlert(altDeleteDescrOK) == 0) {
		    db_delete_description(i);
		    FrmUpdateForm(frmDescription,updDescriptionList);
		}
	    handled = true;
	    break;
	}
	break;
    case frmUpdateEvent:
	if (e->data.frmUpdate.updateCode == updDescriptionList) {
	    description_destroy();
	    description_init();
	    LstDrawList(GetObjectPtr(lstDescription));
	    handled = true;
	}
    default:
        break;
    }

    return handled;
}

