/*   
 *   $Id: float.c,v 1.6 2001/07/11 19:58:11 ondra Exp $
 * 
 *   Holiday Debts manager for PalmOS.
 *   Copyright (C) 2000,2001 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *   You can contact me at 'ondrap@penguin.cz'.
*/

#include <PalmOS.h>
#include "float.h"

static char dPoint = 0;

static void
float_init_dpoint(void)
{
    	/* Set the floating 'point' to '.' or ',' */	
    switch (PrefGetPreference(prefNumberFormat)) {
    case nfCommaPeriod:
    case nfApostrophePeriod:
	dPoint = '.';
	break;
    case nfPeriodComma:
    case nfSpaceComma:
    case nfApostropheComma:
    default:
	dPoint = ',';
	break;
    }
}

double 
float_to_double(Char *str)
{ 
    Int16  i, start, length, punctPos;
    double  result, sign, fractPart;

    result = fractPart = 0.0;
    length = punctPos = StrLen(str);
    
    start = 0;
    sign = 1.0;
    if (str[0] == '-') {
	sign = -1.0;
	start = 1;
    }

    for (i=start; i<length; i++) {    
    // parse the string from left to right converting the integer part
	if (str[i] != '.' && str[i] != ',') {
	    if (str[i]>='0' && str[i]<='9')
		result = result * 10.0 + (str[i] - 0x30);
	    else 
		return NaN;
	}
	else { 
	    punctPos = i;
	    break;
	}
    }
    
    if (str[punctPos] == '.' || str[punctPos] == ',') {
   // parse the string from the end to the '.' converting the fractional part
	for (i=length-1; i>punctPos; i--) {
	    if (str[i]>='0' && str[i]<='9')
		fractPart = fractPart / 10.0 + (str[i] - 0x30);
	    else 
		return NaN;
	}
	result += fractPart / 10.0;
    }
    
    return result * sign; // correcting the sign
}


void 
float_to_str(Char *str, double flpNumber, Int16 numFractDigits)
{ 
    Int32  longNumber;
    double flpIP, zeros, round;
    Int16 i, remainder, strLen;
    Boolean sign = false;

    if (dPoint == 0)
	float_init_dpoint();
    
    flpNumber += 0.000001; /* To evade the -0.00 */

    if (numFractDigits < 0 || numFractDigits > 9)
	return;

    if (flpNumber < 0.0) { 
	flpNumber = -flpNumber;
	sign = true;
    }

    zeros = 1.0;
    for (i=0; i<numFractDigits; i++)
	zeros *= 10;

    round = 0.5/zeros;

// Without MathLib
    flpNumber += round;
    flpIP = (Int32) flpNumber;
    flpNumber = flpNumber - flpIP;

    if (sign) {
	str[0] = '-';
	StrIToA(&str[1], (Int32) flpIP);
    } else 
	StrIToA(&str[0], (Int32) flpIP);
    strLen = StrLen(str);    
    // put in the decimal point and terminate the string
    str[strLen] = dPoint;
    str[numFractDigits+strLen+1] = '\0';

    longNumber = flpNumber * zeros;                       
    // fractional part

    for (i=numFractDigits+strLen; i>strLen; i--) {
	// convert the integer part
	remainder = longNumber % 10;
	str[i] = remainder + 0x30;
	longNumber /= 10;
    }
}

