/*   
 *   $Id: item.c,v 1.28 2002/07/07 17:58:24 ondra Exp $
 * UInt
 *   Holiday Debts manager for PalmOS.
 *   Copyright (C) 2000,2001 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *   You can contact me at 'ondrap@penguin.cz'.
*/

#include <PalmOS.h>

#include "debtsRsc.h"
#include "item.h"
#include "txtask.h"
#include "debts.h"
#include "db.h"
#include "float.h"
#include "currency.h"

static dbPeopleList *peopleList;
static Int16 day,month;
static Int16 year;
static char dateStr[longDateStrLength];
static struct idxArray {
    Int16 *idx;
    Int16 count;
}selected,deselected;
static Int16 payerIdx,defaultPayer;

static Int16 currentItem = -1;
static Int16 selectedLine = -1;
static Int16 defaultCuridx = 0;
/* The last list that was tapped by the user (for correct scrolling) */
static UInt16 lastSelList = 0;

/***********************************************************************
 *
 * FUNCTION:     item_arr_insert
 * 
 * DESCRIPTION:  Insert an item into array
 *
 * PARAMETERS:   item - item to insert
 *               arr - where to insert the item
 *
 * RETURN:       
 *      
 ***********************************************************************/
static Boolean
idx_arr_insert(Int16 item,struct idxArray *arr)
{
    Int16 i,j;

    /* Find position for inserting */
    for (i=0;i < arr->count && item > arr->idx[i];i++)
	;
    if (i < arr->count && arr->idx[i] == item)
	return false; /* Item exists */
    /* Move everything one position forward */
    for (j=arr->count;j>i;j--)
	arr->idx[j] = arr->idx[j-1];

    arr->idx[i] = item;
    arr->count++;

    return true;
}

/***********************************************************************
 *
 * FUNCTION:     item_arr_remove
 * 
 * DESCRIPTION:  Remove item from array
 *
 * PARAMETERS:   item - item from array to remove
 *               arr - from where to remove the item
 *
 * RETURN:       true - item removed
 *               false - item not found
 *      
 ***********************************************************************/
static Boolean
idx_arr_remove(Int16 item,struct idxArray *arr)
{
    Int16 i;

    for (i=0;i<arr->count;i++)
	if (arr->idx[i] == item)
	    break;
    if (i==arr->count)
	return false; /* Item doesn't exist */
    for (i++;i<arr->count;i++)
	arr->idx[i-1] = arr->idx[i];
    arr->count--;
    return true;
}
    
/***********************************************************************
 *
 * FUNCTION:     item_date_display
 * 
 * DESCRIPTION:  Display the selected day on selector
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
item_date_display(void)
{
    DateToAscii(month,day,year,longDateFormat,dateStr);
    CtlSetLabel(GetObjectPtr(selItemDate),dateStr);
}

static void
item_choose_description(void)
{
    char **descr;
    Int16 i,size;
    char *tmp;
    static char editdescr[MAX_INTL_LEN];
    
    /* Internationalization support */
    SysCopyStringResource(editdescr,strEditDescr);
    
    descr = db_get_descriptions();
    for (i=0;descr[i];i++)
	;
    size = i;
    descr[size] = editdescr;
    
    LstSetListChoices(GetObjectPtr(lstItemDescription),descr,size+1);
    LstSetHeight(GetObjectPtr(lstItemDescription),(size+1)>8?8:size+1);

    i = LstPopupList(GetObjectPtr(lstItemDescription));
    if (i == size)
	FrmPopupForm(frmDescription);
    else if (i != noListSelection) {
	tmp = FldGetTextPtr(GetObjectPtr(fldItemDescr));
	if (tmp != NULL)
	    FldDelete(GetObjectPtr(fldItemDescr),0,StrLen(tmp));
	FldInsert(GetObjectPtr(fldItemDescr),descr[i],StrLen(descr[i]));
    }

    descr[size] = NULL;
    db_free_descriptions(descr);
}

/***********************************************************************
 *
 * FUNCTION:     item_list_update
 * 
 * DESCRIPTION:  Update lists to reflect change in 
 *               selected/deselected people
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
item_list_update(void)
{
    LstSetListChoices(GetObjectPtr(lstItemSel),
		      NULL,
		      selected.count);
    LstSetListChoices(GetObjectPtr(lstItemDesel),
		      NULL,
		      deselected.count);
}

/***********************************************************************
 *
 * FUNCTION:     item_list_draw
 * 
 * DESCRIPTION:  Draws a name in bounds, identified by idx[itemnum]
 *
 * PARAMETERS:   itemnum - item number to draw
 *               bounds - bounds of a row
 *               idx - array of index to peopleList->id[],
 *                     user ID is peopleList->id[idx[itemnum]]
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
item_list_draw(Int16 itemnum,RectangleType *bounds,
	       Int16 *idx)
{
    RectangleType rp;

    WinGetClip(&rp);
    WinSetClip(bounds);

    WinDrawChars(peopleList->names[idx[itemnum]],
		 StrLen(peopleList->names[idx[itemnum]]),
		 bounds->topLeft.x, bounds->topLeft.y);

    WinSetClip(&rp);
}

/***********************************************************************
 *
 * FUNCTION:     item_list_draw
 * 
 * DESCRIPTION:  Draws a coeficient of a specified item (user)
 *
 * PARAMETERS:   itemnum - item number to draw
 *               bounds - bounds of a row
 *               idx - array of index to peopleList->id[],
 *                     user ID is peopleList->id[idx[itemnum]]
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
item_list_draw_coef(Int16 itemnum, RectangleType *bounds,
		    Int16 *idx)
{
    char coefstr[15];
    RectangleType rp;
    Int16 i,j,drawx;

    /* Do not draw the default 1.0 coeficient */
    if (peopleList->people[idx[itemnum]].coef == 1.0)
	return;

    WinGetClip(&rp);
    WinSetClip(bounds);

    float_to_str(coefstr,peopleList->people[idx[itemnum]].coef,1);
    for (i=0;coefstr[i];i++)
	if (coefstr[i] == '.' || coefstr[i] == ',') {
	    for (j=StrLen(coefstr)-1;j>=i;j--) {
		if (coefstr[j] != '0' && coefstr[j] != ',' &&
		    coefstr[j] != '.')
		    break;
		coefstr[j] = '\0';
	    }
	    break;
	}
    drawx = bounds->topLeft.x + bounds->extent.x - 1 -
	FntCharsWidth(coefstr,StrLen(coefstr));
    WinDrawChars(coefstr,StrLen(coefstr),drawx,bounds->topLeft.y);

    WinSetClip(&rp);
}
/***********************************************************************
 *
 * FUNCTION:     item_list_draw_sel
 * 
 * DESCRIPTION:  Callback draw function for list of selected people,
 *               calls drawing function for People
 *
 * PARAMETERS:   See PalmOS reference
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
item_list_draw_sel(Int16 itemnum,RectangleType *bounds,
		   Char **tmp)
{
    item_list_draw(itemnum,bounds,selected.idx);
    item_list_draw_coef(itemnum,bounds,selected.idx);
}

/***********************************************************************
 *
 * FUNCTION:     item_list_draw_desel
 * 
 * DESCRIPTION:  Callback draw function for list of deselected people,
 *               calls drawing function for People
 *
 * PARAMETERS:   See PalmOS reference
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
item_list_draw_desel(Int16 itemnum,RectangleType *bounds,
		   Char **tmp)
{
    item_list_draw(itemnum,bounds,deselected.idx);
}

/***********************************************************************
 *
 * FUNCTION:     item_save_ok
 * 
 * DESCRIPTION:  Checks entered parameters and saves/replaces the item
 *               in the database
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       true - item saved
 *               false - item not saved, error in input
 *      
 ***********************************************************************/
static Boolean
item_save_ok(void)
{
    itemRec item;
    Int16 i;

    item.date.year = year - 1904;
    item.date.month = month;
    item.date.day = day;

    if (!FldGetTextPtr(GetObjectPtr(fldItemAmount)) || \
	!StrLen(FldGetTextPtr(GetObjectPtr(fldItemAmount)))) {
//	FrmAlert(altMissingAmount);
//	return false;
	item.amount = 0.0;
    } else 
	item.amount = float_to_double(FldGetTextPtr(GetObjectPtr(fldItemAmount)));
    if (item.amount == NaN) {
	FrmAlert(altMissingAmount);
	return false;
    }

    if (payerIdx == -1) {
	FrmAlert(altMissingPayer);
	return false;
    }
    item.payer = peopleList->people[payerIdx].id;

    item.currency = currencyList->list[LstGetSelection(GetObjectPtr(lstItemCurrency))]->id;
    item.transfer = CtlGetValue(GetObjectPtr(chkItemTransfer));

    if (!FldGetTextPtr(GetObjectPtr(fldItemDescr)))
	item.description[0] = '\0';
    else
	StrCopy(item.description,FldGetTextPtr(GetObjectPtr(fldItemDescr)));

    if (selected.count == 0) {
	FrmAlert(altMissingPayee);
	return false;
    }
    item.npayees = selected.count;

    item.pmts = MemPtrNew(peopleList->nitems * sizeof(item.pmts[0]));
    for (i=0;i<selected.count;i++) {
	item.pmts[i].id = peopleList->people[selected.idx[i]].id;
	item.pmts[i].coef = peopleList->people[selected.idx[i]].coef;
    }

    if (currentItem != -1) 
	db_delete_item(currentItem);
    db_add_item(&item);
    
    MemPtrFree(item.pmts);
    
    return true;
}

/***********************************************************************
 *
 * FUNCTION:     item_init
 * 
 * DESCRIPTION:  Constructor for itemForm
 *               - if currentItem is not -1, fill the fields 
 *                 with appropriate record
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
void
item_init(void)
{
    DateType date;
    Int16 i,j;

    peopleList = db_get_peoplelist();

    LstSetListChoices(GetObjectPtr(lstItemCurrency),
		      (char **)currencyList->list,currencyList->nitems);
    LstSetHeight(GetObjectPtr(lstItemCurrency),
		 currencyList->nitems > 5 ? 5 : currencyList->nitems);

    LstSetListChoices(GetObjectPtr(lstItemPayer),
		      peopleList->names,peopleList->nitems);
    LstSetHeight(GetObjectPtr(lstItemPayer),
		 peopleList->nitems > 5 ? 5 : peopleList->nitems);

    if (peopleList->nitems > 0) {
	selected.idx = MemPtrNew(peopleList->nitems * sizeof(selected.idx[0]));
	deselected.idx = MemPtrNew(peopleList->nitems * sizeof(selected.idx[0]));
    } else {
	selected.idx = NULL;
	deselected.idx = NULL;
    }

    /* Fill the deselected field and empty the selected field */
    selected.count = 0;
    for (i=0;i<peopleList->nitems;i++) 
	deselected.idx[i] = i;
    deselected.count = peopleList->nitems;
    selected.count = 0;

    if (currentItem != -1) { /* Set up fields according existing item */
	itemType *item;
	char amountstr[15];

	item = db_get_item(currentItem);

	day = item->date.day;
	month = item->date.month;
	year = item->date.year + 1904;

	FldInsert(GetObjectPtr(fldItemDescr),item->description,
		  StrLen(item->description));

	for (i=0;i<item->npayees;i++) {
	    for (j=0;j < peopleList->nitems && peopleList->people[j].id != item->pmts[i].id;j++)
		;
	    if (j == peopleList->nitems)
		continue;
	    peopleList->people[j].coef = item->pmts[i].coef;
	    idx_arr_insert(j,&selected);
	    idx_arr_remove(j,&deselected);
	}

	CtlSetValue(GetObjectPtr(chkItemTransfer),item->transfer);

	for (i=0;peopleList->people[i].id != item->payer;i++) 
	    ;
	payerIdx = i;
	CtlSetLabel(GetObjectPtr(popItemPayer),peopleList->names[payerIdx]);
	LstSetSelection(GetObjectPtr(lstItemPayer),i);

	for (i=0;currencyList->list[i]->id != item->currency;i++)
	    ;
	CtlSetLabel(GetObjectPtr(popItemCurrency),currencyList->list[i]->name);
	LstSetSelection(GetObjectPtr(lstItemCurrency),i);

	if (item->amount != 0.0) {
	    float_to_str(amountstr,item->amount,2);
	    FldInsert(GetObjectPtr(fldItemAmount),amountstr,
		      StrLen(amountstr));
	}

	db_free_item(item);
    } else { /* New item */
        /* Hide the delete button for new item */
	FrmHideObject(FrmGetActiveForm(),
		      FrmGetObjectIndex(FrmGetActiveForm(),btnItemDelete));
	/* Set date to today */
	DateSecondsToDate(TimGetSeconds(),&date);
	day = date.day;
	month = date.month;
	year = date.year + 1904;
	
	CtlSetLabel(GetObjectPtr(popItemCurrency),
		    currencyList->list[defaultCuridx]->name);
	LstSetSelection(GetObjectPtr(lstItemCurrency),defaultCuridx);

	if (defaultPayer == -1)
	    payerIdx = -1;
	else {
	    for (i=0;peopleList->people[i].id != defaultPayer;i++) 
		;
	    payerIdx = i;
	    CtlSetLabel(GetObjectPtr(popItemPayer),
			peopleList->names[payerIdx]);
	    LstSetSelection(GetObjectPtr(lstItemPayer),i);
	}
    }
    item_date_display();

    LstSetDrawFunction(GetObjectPtr(lstItemSel), item_list_draw_sel);
    LstSetDrawFunction(GetObjectPtr(lstItemDesel), item_list_draw_desel);
    item_list_update();

    selectedLine = -1;
    lastSelList = 0;
}

/***********************************************************************
 *
 * FUNCTION:     item_destroy
 * 
 * DESCRIPTION:  Destructor for formItem
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
item_destroy(void)
{
    db_free_peoplelist(peopleList);
    if (selected.idx != NULL)
	MemPtrFree(selected.idx);
    if (deselected.idx != NULL)
	MemPtrFree(deselected.idx);

    selected.idx = NULL;
    deselected.idx = NULL;
}

/***********************************************************************
 *
 * FUNCTION:     ItemFormHandleEvent
 * 
 * DESCRIPTION:  Event handler for formItem
 *
 * PARAMETERS:   event
 *
 * RETURN:       handled
 *      
 ***********************************************************************/
Boolean 
ItemFormHandleEvent (EventPtr e)
{
    Boolean handled = false;
    FormPtr frm;
    ListPtr list;
    Int16 i;
    UInt16 focusobj;
    char *t;
    
    switch (e->eType) {
    case keyDownEvent:
	if (e->data.keyDown.chr == pageDownChr ||
	    e->data.keyDown.chr == pageUpChr) {
	    if (lastSelList == 0)
		break;
	    
	    list = GetObjectPtr(lastSelList);
	    frm = FrmGetActiveForm();

	    FrmHideObject(frm,FrmGetObjectIndex(frm,lastSelList));
	    LstScrollList(list,
			  e->data.keyDown.chr==pageUpChr ? winUp : winDown,
			  LstGetVisibleItems(list)-1);
	    FrmShowObject(frm,FrmGetObjectIndex(frm,lastSelList));
	    
	    handled = true;
	    break;
	}

	focusobj = FrmGetFocus(FrmGetActiveForm());
	if (focusobj != FrmGetObjectIndex(FrmGetActiveForm(),fldItemAmount))
	    return false;
	/* Emulate numeric field for amount with allowed negative */
	switch (e->data.keyDown.chr) {
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case leftArrowChr:
	case rightArrowChr:
	case chrBackspace:
	    return false;
	case '-':
	    if (FldGetInsPtPosition(GetObjectPtr(fldItemAmount)) == 0)
		return false;
	    else
		return true;
	    break;
	case '.':
	case ',':
	    t = FldGetTextPtr(GetObjectPtr(fldItemAmount));
	    if (!t)
		return false;
	    else {
		/* Discard if already written */
		if (StrChr(t,e->data.keyDown.chr))
		    return true;
		/* Allow only if match national settings */
		if (e->data.keyDown.chr == '.') {
		    switch (PrefGetPreference(prefNumberFormat)) {
		    case nfCommaPeriod:
		    case nfApostrophePeriod:
			return false;
		    default:
			return true;
		    }
		} else {
		    switch (PrefGetPreference(prefNumberFormat)) {
		    case nfCommaPeriod:
		    case nfApostrophePeriod:
			return true;
		    default:
			return false;
		    }
		}
	    }
	    break;
	default:
	    return true;
	}
	break;
    case frmOpenEvent:
	frm = FrmGetActiveForm();
	item_init();
	FrmDrawForm(frm);
	FrmSetFocus(frm,FrmGetObjectIndex(frm,fldItemAmount));
	handled = true;
	break;
    case frmCloseEvent:
	item_destroy();
	handled = false;
	break;
    case frmUpdateEvent:
	if (e->data.frmUpdate.updateCode == updPeopleList) {
	    handled = true;
	    item_list_update();
	    LstDrawList(GetObjectPtr(lstItemDesel));
	    LstDrawList(GetObjectPtr(lstItemSel));
	    selectedLine = -1;
	}
	break;
    case popSelectEvent:
	switch (e->data.popSelect.listID) {
	case lstItemPayer:
	    payerIdx = e->data.popSelect.selection;
	    break;
	}
	handled = false;
	break;
    case lstSelectEvent:
	/* Set the ID of last selected list */
	if (e->data.lstSelect.listID == lstItemSel ||
	    e->data.lstSelect.listID == lstItemDesel)
	    lastSelList = e->data.lstSelect.listID;

	if (e->data.lstSelect.listID == lstItemSel) {
	    i = e->data.lstSelect.selection;
	    if (selectedLine == i) {
		if (txtask_dialog_num(strCoef,strCoefLbl,
				      &peopleList->people[selected.idx[i]].coef)) 
		    FrmUpdateForm(frmItem,updPeopleList);
	    } else
		selectedLine = e->data.lstSelect.selection;
	}
	handled = true;
	break;
    case ctlSelectEvent:
	switch(e->data.ctlSelect.controlID) {
	case popItemDescription:
	    item_choose_description();
	    handled = true;
	    break;
	case btnItemSelAll:
	    deselected.count = 0;
	    for (i=0;i<peopleList->nitems;i++)
		selected.idx[i] = i;
	    selected.count = peopleList->nitems;
	    FrmUpdateForm(frmItem,updPeopleList);
	    handled = true;
	    break;
	case btnItemDeselAll:
	    selected.count = 0;
	    for (i=0;i<peopleList->nitems;i++)
		deselected.idx[i] = i;
	    deselected.count = peopleList->nitems;
	    FrmUpdateForm(frmItem,updPeopleList);
	    handled = true;
	    break;
	case btnItemSelOne:
	    i = LstGetSelection(GetObjectPtr(lstItemDesel));
	    if (i != noListSelection) {
		idx_arr_insert(deselected.idx[i],&selected);
		idx_arr_remove(deselected.idx[i],&deselected);
		FrmUpdateForm(frmItem,updPeopleList);
	    }
	    handled = true;
	    break;
	case btnItemDeselOne:
	    i = LstGetSelection(GetObjectPtr(lstItemSel));
	    if (i != noListSelection) {
		idx_arr_insert(selected.idx[i],&deselected);
		idx_arr_remove(selected.idx[i],&selected);
		FrmUpdateForm(frmItem,updPeopleList);
	    }
	    handled = true;
	    break;
	case btnItemDelete:
	    if (currentItem != -1 && FrmAlert(altDeleteItemOK)==0) {
		db_delete_item(currentItem);
		item_destroy();
		FrmReturnToForm(0);
		FrmUpdateForm(frmMain,updMainTable);
	    }
	    handled = true;
	    break;
	case btnItemOK:
	    if (item_save_ok()) {
		item_destroy();
		FrmReturnToForm(0);
		FrmUpdateForm(frmMain,updMainTable);
	    }
	    handled = true;
	    break;
	case btnItemCancel:
	    item_destroy();
	    FrmReturnToForm(0);
	    handled = true;
	    break;
	case selItemDate:
	    if (SelectDay(selectDayByDay,&month,&day,&year,"Select Day"))
		item_date_display();
	    handled = true;
	    break;
	}
	break;
    default:
        break;
    }

    return handled;
}

/***********************************************************************
 *
 * FUNCTION:     item_popup_new
 * 
 * DESCRIPTION:  Popup a formItem to create new record
 *
 * PARAMETERS:   payer - preselected payr
 *
 * RETURN:       Nothing 
 *      
 ***********************************************************************/
void
item_popup_new(Int16 payer,Int16 curidx)
{
    currentItem = -1;
    defaultPayer = payer;
    if (curidx == -1)
	defaultCuridx = 0;
    else
	defaultCuridx = curidx;
    FrmPopupForm(frmItem);
}

/***********************************************************************
 *
 * FUNCTION:     item_popup_change
 * 
 * DESCRIPTION:  Popup a formitem to edit an existing record
 *
 * PARAMETERS:   idx - index of existing record
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
void 
item_popup_change(Int16 idx)
{
    currentItem = idx;
    FrmPopupForm(frmItem);
}

