/*   
 *   $Id: main.c,v 1.25 2002/07/12 19:30:32 ondra Exp $
 * 
 *   Holiday Debts manager for PalmOS.
 *   Copyright (C) 2000,2001 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *   You can contact me at 'ondrap@penguin.cz'.
*/

#include <PalmOS.h>

#include "debtsRsc.h"
#include "db.h"
#include "item.h"
#include "float.h"
#include "main.h"
#include "debts.h"
#include "currency.h"
#include "txtask.h"

static dbItemList *itemList=NULL;
static Int16 firstLine;
static Int16 filterUser = -1;
Int16 filterCuridx = -1;
static dbPeopleList *peopleList=NULL;
static char **filterMenu = NULL;
static CostMode costMode=TOTALCOST;
static LocalID dbId;
static UInt16 cardNo;
static char **filterCurMenu = NULL;
dbPrefs dbprefs;

/***********************************************************************
 *
 * FUNCTION:     main_table_draw
 * 
 * DESCRIPTION:  Draw an item in the table
 *
 * PARAMETERS:   See PalmOS Reference
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
main_table_draw(void *table,Int16 row,Int16 column,
		RectangleType *bounds)
{
    itemType *item;
    char datebuffer[dateStringLength];
    char amountstr[15];
    char *datestr;
    Int16 i;
    Int16 rowid;
    Int16 drawx,drawy;
    FontID oldfont=0; /* make gcc happy */

    rowid = TblGetRowID(table,row);

    item = db_get_item(rowid);

    /* Draw the due date */
    DateToAscii(item->date.month,item->date.day,item->date.year + 1904,
		dateFormat,datebuffer);

    /* Remove the year from string */
    datestr = datebuffer;
    if ((dateFormat == dfYMDWithSlashes) ||
	(dateFormat == dfYMDWithDots) ||
	(dateFormat == dfYMDWithDashes))
	datestr += 3;
    else
	datestr[StrLen(datestr) - 3] = 0;
    
    drawx = 1;
    drawy = bounds->topLeft.y;
    
    WinDrawChars(datestr,StrLen(datestr),drawx,drawy);
    
    /* Draw the description, bold if transfer */
    drawx = 29;
    
    if (item->transfer)
	    oldfont = FntSetFont(boldFont);

    if (palmOS31) 
	WinDrawTruncChars(item->description,StrLen(item->description),
			  drawx,drawy,85);
    else {
		RectangleType rp,rp1;
		
		WinGetClip(&rp);
		rp1 = *bounds;
		rp1.topLeft.x = 1;
		rp1.extent.x = 85;
		WinSetClip(&rp1);
		WinDrawChars(item->description,
			     StrLen(item->description),drawx,drawy);
		WinSetClip(&rp);
    }

    if (item->transfer)
	    FntSetFont(oldfont);

    /* Draw the amount */
    if (item->amount == 0.0) 
	StrCopy(amountstr,"XXXXX");
    else {
	float_to_str(amountstr,get_item_costs(item,filterUser,
					      costMode,filterCuridx),2);
	if (filterCuridx != -1) {
	    StrCat(amountstr,currencyList->list[filterCuridx]->name);
	} else {
	    for (i=0;currencyList->list[i]->id != item->currency;i++)
		;
	    StrCat(amountstr,currencyList->list[i]->name);
	}
    }

    drawx = bounds->topLeft.x + bounds->extent.x - 
	FntCharsWidth(amountstr,StrLen(amountstr));
    WinDrawChars(amountstr,StrLen(amountstr),drawx,drawy);

    db_free_item(item);
}

/***********************************************************************
 *
 * FUNCTION:     main_table_pendown
 * 
 * DESCRIPTION:  Handle the tap on a cell of a table
 *
 * PARAMETERS:   bounds of a cell
 *
 * RETURN:       true - user selected the cell
 *               false - user went outside cell and lifted the pen
 *      
 ***********************************************************************/
static Boolean
main_table_pendown(RectangleType *bounds)
{
    Boolean penDown;
    Boolean inrect = true;
    Coord x,y;
	
    WinInvertRectangle(bounds,0);
    do {
	EvtGetPen(&x,&y,&penDown);
	if (RctPtInRectangle(x,y,bounds) && !inrect) {
	    inrect = true;
	    WinInvertRectangle(bounds,0);
	} else if (!RctPtInRectangle(x,y,bounds) && inrect) {
	    inrect = false;
	    WinInvertRectangle(bounds,0);
	}		
    }while(penDown);
    if (inrect)
	WinInvertRectangle(bounds,0);
    return inrect;
}

/***********************************************************************
 *
 * FUNCTION:     main_update_label
 * 
 * DESCRIPTION:  Changes the label to 'text' + 'num' + 'currencyname'
 *
 * PARAMETERS:   id - id of the label
 *               text - text part of a new label
 *               num - numeric part of label
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
main_update_label(UInt16 id,char *text,double num)
{
    char strbuf[50];
    char numbuf[20];
    FormPtr frm = FrmGetActiveForm();

    StrCopy(strbuf,text);
    StrCat(strbuf," ");
    float_to_str(numbuf,num,2);
    StrCat(strbuf,numbuf);
    if (filterCuridx == -1) 
	StrCat(strbuf,currencyList->list[0]->name);
    else
	StrCat(strbuf,currencyList->list[filterCuridx]->name);
    FrmHideObject(frm,FrmGetObjectIndex(frm,id));
    FrmCopyLabel(frm,id,strbuf);
    FrmShowObject(frm,FrmGetObjectIndex(frm,id));
}

/***********************************************************************
 *
 * FUNCTION:     main_update_counters
 * 
 * DESCRIPTION:  Update the onscreen 'Costs' and 'Balance' labels
 *               Called when there is a change in the database or
 *               in filtering of items
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
main_update_counters(void)
{
    char tmptxt[MAX_INTL_LEN];
    double amount;
    
    if (costMode == USERPAYMENT) {
	SysCopyStringResource(tmptxt, strPaymentLbl);
	if (filterCuridx != -1)
	    amount = db_compute_payment(filterUser,filterCuridx);
	else
	    amount = db_compute_payment(filterUser,0);
    } else {
	StrCopy(tmptxt,"   ");
	SysCopyStringResource(tmptxt+3, strCosts);
	if (filterCuridx != -1)
	    amount = db_compute_cost(filterUser,filterCuridx);
	else
	    amount = db_compute_cost(filterUser,0);
    }
    main_update_label(lblMainCost,tmptxt,amount);

    SysCopyStringResource(tmptxt,strBalance);
    if (filterUser != -1 && filterCuridx != -1)
	main_update_label(lblMainBalance,tmptxt,
			  db_compute_balance(filterUser,filterCuridx));
    else if (filterUser != -1) 
	main_update_label(lblMainBalance,tmptxt,
			  db_compute_balance(filterUser,0));
    else
	FrmHideObject(FrmGetActiveForm(),
		      FrmGetObjectIndex(FrmGetActiveForm(),lblMainBalance));
}

/***********************************************************************
 *
 * FUNCTION:     main_update_items
 * 
 * DESCRIPTION:  Handle change in database/filtering 
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
main_update_items(void)
{
    if (itemList != NULL) {
	db_free_itemlist(itemList);
	itemList = NULL;
    }
    itemList = db_get_itemlist(filterUser);
    main_update_counters();
}

/***********************************************************************
 *
 * FUNCTION:     main_init_table
 * 
 * DESCRIPTION:  Initialize the main table
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
main_init_table(void)
{
    UInt16 rows;
    TablePtr table;
    Int16 i;
    
    table = GetObjectPtr(tblMain);
    rows = TblGetNumberOfRows(table);

    for (i=0;i<rows;i++) 
	TblSetItemStyle(table,i,0,customTableItem);

    TblSetColumnUsable(table,0,true);

    TblSetCustomDrawProcedure(table,0,main_table_draw);
}

/***********************************************************************
 *
 * FUNCTION:     main_load_table
 * 
 * DESCRIPTION:  Load the table with appropriate values
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
main_load_table(void)
{
    TablePtr table;
    Int16 i,j;
    UInt16 rows;

    table = GetObjectPtr(tblMain);
    rows = TblGetNumberOfRows(table);

    if (firstLine > itemList->nitems - rows)
	firstLine = itemList->nitems - rows;
    if (itemList->nitems <= rows)
	firstLine = 0;
    
    for (i=0,j=firstLine;i<rows && j<itemList->nitems;i++,j++) {
	TblSetRowID(table,i,itemList->seq[j]);
	TblSetRowUsable(table,i,true);
    }

    for (;i<rows;i++) 
	TblSetRowUsable(table,i,false);
}

/***********************************************************************
 *
 * FUNCTION:     main_update_scrollbar
 * 
 * DESCRIPTION:  Updates the scrollbar to reflect reality
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
void
main_update_scrollbar(void)
{
    TablePtr table;
    UInt16 rows;

    table = GetObjectPtr(tblMain);
    rows = TblGetNumberOfRows(table);

    if (firstLine == 0 && itemList->nitems <= rows) 
	SclSetScrollBar(GetObjectPtr(sclMain),firstLine,0,0,0);
    else 
	SclSetScrollBar(GetObjectPtr(sclMain),firstLine,0,
			itemList->nitems - rows,
			rows);
}

/***********************************************************************
 *
 * FUNCTION:     main_table_scroll
 * 
 * DESCRIPTION:  If possible, scroll the table 'delta' rows
 *
 * PARAMETERS:   table - pointer to table to scroll
 *               delta - how many rows and direction (+/-)
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
void
main_table_scroll(TablePtr table,Int16 delta)
{
    RectangleType r;
    UInt16 i,rows;
    Int16 oldfirstline;

    oldfirstline = firstLine;
    firstLine +=delta;
    if (firstLine < 0)
	firstLine = 0;
    main_load_table();
    delta = firstLine - oldfirstline;
 
    if (delta > 0)
	i = delta;
    else
	i = -delta;
    
    rows = TblGetNumberOfRows(table);
    if (i < rows) {
	TblGetBounds(table,&r);
	WinScrollRectangle(&r, (delta < 0) ? winDown : winUp, 
			   TblGetRowHeight(table, 0) * i, &r);
	
	while (i--) 
	    TblMarkRowInvalid(table, (delta < 0) ? i : rows-i-1);
    } else
	TblMarkTableInvalid(table);

    TblRedrawTable(table);
}

/***********************************************************************
 *
 * FUNCTION:     main_update_cost
 * 
 * DESCRIPTION:  Update the 'cost mode' button to reflect
 *               a change
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
main_update_cost(void)
{
    ListPtr list;
    
    list = GetObjectPtr(lstMainCost);
    LstSetSelection(list,costMode);
    CtlSetLabel(GetObjectPtr(btnMainCost),LstGetSelectionText(list,costMode));
}

/***********************************************************************
 *
 * FUNCTION:     main_update_people
 * 
 * DESCRIPTION:  Reflect the change in people database
 *               - reselct the right person in the filter menu
 *               - if the selected user was deleted, change to 'All' filter
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
main_update_people(void)
{
    Int16 i;
    static char alltxt[MAX_INTL_LEN];

    SysCopyStringResource(alltxt, strAll);

    if (peopleList != NULL) {
	db_free_peoplelist(peopleList);
	peopleList = NULL;
    }
    
    if (filterMenu != NULL) {
	MemPtrFree(filterMenu);
	filterMenu = NULL;
    }

    peopleList = db_get_peoplelist();
    filterMenu = MemPtrNew(sizeof(filterMenu[0]) * (peopleList->nitems+1));
    filterMenu[0] = alltxt;
    for (i=0;i<peopleList->nitems;i++)
	filterMenu[i+1] = peopleList->names[i];
    LstSetListChoices(GetObjectPtr(lstMainFilter),
		      filterMenu,peopleList->nitems+1);
    if (peopleList->nitems > 9)
	LstSetHeight(GetObjectPtr(lstMainFilter),10);
    else
	LstSetHeight(GetObjectPtr(lstMainFilter),peopleList->nitems+1);

    for (i=0;i<peopleList->nitems;i++)
	if (peopleList->people[i].id == filterUser) {
	    LstSetSelection(GetObjectPtr(lstMainFilter),i+1);
	    CtlSetLabel(GetObjectPtr(popMainFilter),peopleList->names[i]);
	    break;
	}
    if (i==peopleList->nitems) {
	CtlSetLabel(GetObjectPtr(popMainFilter),alltxt);
	LstSetSelection(GetObjectPtr(lstMainFilter),0);
	if (filterUser != -1) {
	    /* The selected user was deleted, force redraw */
	    filterUser = -1;
	    FrmUpdateForm(frmMain,updMainTable);
	}
    }
}

static void
main_update_currency()
{
    Int16 i;

    if (filterCurMenu) {
	MemPtrFree(filterCurMenu);
	filterCurMenu = NULL;
    }

    filterCurMenu = MemPtrNew(sizeof(char *) * (currencyList->nitems+1));
    filterCurMenu[0] = "--";
    for (i=0;i < currencyList->nitems;i++)
	filterCurMenu[i+1] = (char *)currencyList->list[i];
    
    LstSetListChoices(GetObjectPtr(lstMainCurrency),
		      filterCurMenu,currencyList->nitems+1);
    LstSetHeight(GetObjectPtr(lstMainCurrency),currencyList->nitems+1);

    if (filterCuridx >= currencyList->nitems) {
	filterCuridx = -1;
	FrmUpdateForm(frmMain,updMainTable);
    }

    LstSetSelection(GetObjectPtr(lstMainCurrency),filterCuridx+1);
    CtlSetLabel(GetObjectPtr(popMainCurrency),filterCurMenu[filterCuridx+1]);
}

/***********************************************************************
 *
 * FUNCTION:     main_init
 * 
 * DESCRIPTION:  Basic constructor for the main form
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
main_init()
{
    static char dbname[dmDBNameLength+1];
    
    db_get_name(dbname);
    FrmSetTitle(FrmGetActiveForm(),dbname+4);

    main_update_items();
    main_init_table();
    main_load_table();
    main_update_scrollbar();
    main_update_people();
    main_update_currency();
    main_update_cost();
}

/***********************************************************************
 *
 * FUNCTION:     main_destroy
 * 
 * DESCRIPTION:  Destructor for the main form
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
main_destroy(void)
{
    if (itemList) {
	db_free_itemlist(itemList);
	itemList = NULL;
    }
    if (peopleList) {
	db_free_peoplelist(peopleList);
	peopleList = NULL;
    }
    if (filterMenu) {
	MemPtrFree(filterMenu);
	filterMenu = NULL;
    }
    if (filterCurMenu) {
	MemPtrFree(filterCurMenu);
	filterCurMenu = NULL;
    }
}

/***********************************************************************
 *
 * FUNCTION:     MainFormHandleEvent
 * 
 * DESCRIPTION:  Event Handler for the main form
 *
 * PARAMETERS:   Event
 *
 * RETURN:       handled
 *      
 ***********************************************************************/
Boolean 
MainFormHandleEvent (EventPtr e)
{
    Boolean handled = false;
    FormPtr frm;
    Int16 row,oldrow,col,delta,i;
    RectangleType bounds;
	
    switch (e->eType) {
    case frmOpenEvent:
	frm = FrmGetActiveForm();

	if (!db_open_by_id(dbId,cardNo)) {
	    FrmGotoForm(frmDBmgr);
	    handled = true;
	    break;
	}
	dbprefs = db_get_prefs();
	filterUser = dbprefs.fuser;
	filterCuridx = dbprefs.curidx;
	firstLine = dbprefs.firstline;
	costMode = dbprefs.costMode;

	cur_init_list();
	main_init();
    
	FrmDrawForm(frm);

	if (peopleList->nitems == 0)
	    FrmPopupForm(frmPeople);
	handled = true;
	break;
    case frmSaveEvent:
	debts_save_main(dbId,cardNo);
	handled = true;
	break;
    case frmCloseEvent:
	dbprefs.curidx = filterCuridx;
	dbprefs.fuser = filterUser;
	dbprefs.firstline = firstLine;
	dbprefs.costMode = costMode;
	db_save_prefs(dbprefs);

	main_destroy();
	cur_destroy_list();
	db_close();
	handled = false;
	break;
    
    case popSelectEvent:
	switch (e->data.popSelect.listID) {
	case lstMainCurrency:
	    filterCuridx = e->data.popSelect.selection - 1;
	    FrmUpdateForm(frmMain,updMainTable);
	    break;
	case lstMainFilter:
	    if (e->data.popSelect.selection != e->data.popSelect.priorSelection) {
		row = e->data.popSelect.selection;
		oldrow = e->data.popSelect.priorSelection;
		if (row == 0) {
		    filterUser = -1;
		    costMode = TOTALCOST;
		    main_update_cost();
		} else {
		    filterUser = peopleList->people[row-1].id;
		    if (oldrow == 0) {
			costMode = USERCOST;
			main_update_cost();
		    }
		}
		firstLine = 0;
		FrmUpdateForm(frmMain,updMainTable);
	    }
	    break;
	}
	handled = false;
	break;
	
    case keyDownEvent:
	switch (e->data.keyDown.chr) {
	case pageDownChr:
	    main_table_scroll(GetObjectPtr(tblMain),9);
	    main_update_scrollbar();
	    handled = true;
	    break;
	case pageUpChr:
	    main_table_scroll(GetObjectPtr(tblMain),-9);
	    main_update_scrollbar();
	    handled = true;
	    break;
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	    item_popup_new(filterUser,filterCuridx);
	    EvtAddEventToQueue(e);
	    handled = true;
	    break;
	}
	break;
    case sclRepeatEvent:
	delta = e->data.sclRepeat.newValue - e->data.sclRepeat.value;
	main_table_scroll(GetObjectPtr(tblMain),delta);
	handled = false;
	break;

    case tblEnterEvent:
	row = e->data.tblSelect.row;
	col = e->data.tblSelect.column;
	TblGetItemBounds(GetObjectPtr(tblMain),
			 row,col,&bounds);
	if (main_table_pendown(&bounds)) 
	    item_popup_change(TblGetRowID(e->data.tblSelect.pTable,row));

	handled = true;
	break;
    case menuEvent:
	MenuEraseStatus(NULL);

	switch(e->data.menu.itemID) {
	case menuHelp:
	    FrmHelp(strHelp);
	    handled = true;
	    break;
	case menuMainPrefs:
	    if (txtask_dialog_cbox(strDBPreferences,strDefaultCurrency,
				   strIndirect,
				   dbprefs.lcurrency,MAX_CURRENCY_NAME,
				   &dbprefs.indirectcur)) {
		cur_init_list();
		FrmUpdateForm(frmMain,frmRedrawUpdateCode);
	    }
	    handled = true;
	    break;
	case menuMainCurrency:
	    FrmPopupForm(frmCurrency);
	    handled = true;
	    break;
	case menuMainPeople:
	    FrmPopupForm(frmPeople);
	    handled = true;
	    break;
	case menuDBAbout:
	    FrmAlert(altAbout);
	    handled = true;
	    break;
	}

    	handled = true;
	break;
    case frmUpdateEvent:
	if (e->data.frmUpdate.updateCode == updMainTable) {
	    main_update_items();
	    main_load_table();
	    TblMarkTableInvalid(GetObjectPtr(tblMain));
	    TblRedrawTable(GetObjectPtr(tblMain));
	    main_update_scrollbar();
	    handled = true;
	} else if (e->data.frmUpdate.updateCode == updPeopleList) {
	    main_update_people();
	    handled = true;
	} else if (e->data.frmUpdate.updateCode == updCurrencyList) {
	    main_update_currency();
	    FrmUpdateForm(frmMain,updMainTable);
	    handled = true;
	} else {
	    main_update_currency(); /* It is possible that we got
				       this redraw request BEFORE 
				       updCurrencyList and the list
				       of names is not more valid */
	    main_update_items();
	    main_load_table();
	}
	break;
    case ctlSelectEvent:
	switch(e->data.ctlSelect.controlID) {
	case btnMainDone:
	    FrmGotoForm(frmDBmgr);
	    handled = true;
	    break;
	case btnMainNew:
	    item_popup_new(filterUser,filterCuridx);
	    handled = true;
	    break;
	case btnMainCost:
	    if (filterUser!=-1) {
		i = LstPopupList(GetObjectPtr(lstMainCost));
		if (i != noListSelection) {
		    costMode = i;
		    main_update_cost();
		    FrmUpdateForm(frmMain,updMainTable);
		}
	    }
	}
	break;
    default:
        break;
    }

    return handled;
}


void
main_opendb(LocalID id,UInt16 cardno)
{
    if (!db_check_database(id,cardno)) {
	FrmGotoForm(frmDBmgr);
	return;
    }
    dbId = id;
    cardNo = cardno;
    FrmGotoForm(frmMain);
}


