/*   
 *   $Id: people.c,v 1.19 2002/07/07 17:58:24 ondra Exp $
 * 
 *   Holiday Debts manager for PalmOS.
 *   Copyright (C) 2000,2001 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *   You can contact me at 'ondrap@penguin.cz'.
*/

#include <PalmOS.h>

#include "debtsRsc.h"
#include "debts.h"
#include "people.h"
#include "db.h"
#include "float.h"
#include "txtask.h"
#include "currency.h"
#include "main.h"

static dbPeopleList *peopleList;
static struct {
    double costs;
    double balance;
}*reportList;

/***********************************************************************
 *
 * FUNCTION:     person_save_ok
 * 
 * DESCRIPTION:  Selects an ID for the person (max(existing)) and
 *               saves person in database
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       true - save ok, false - user supplied bad name
 *      
 ***********************************************************************/
static Boolean
person_change(Int16 curidx)
{
    Int16 id,i;
    char name[MAX_NAME+1];
    Boolean selected;
    double coef;

    if (curidx == -1) {
	StrCopy(name,"");
	coef = 1.0;
    } else {
	StrCopy(name,peopleList->names[curidx]);
	coef = peopleList->people[curidx].coef;
    }

    selected = txtask_dialog_person(name,MAX_NAME,&coef);

    if (!selected)
	return false;

    if (curidx == -1) {
	for (id=1,i=0;i<peopleList->nitems;i++) 
	    if (peopleList->people[i].id >= id)
		id = peopleList->people[i].id + 1;
    }
    else {
	id = peopleList->people[curidx].id;
    }

    db_add_person(id,name,coef);
    return true;
}

/***********************************************************************
 *
 * FUNCTION:     people_draw_item
 * 
 * DESCRIPTION:  Custom drawing procedure for PeopleList
 *
 * PARAMETERS:   See PalmOS Reference
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
people_draw_item(Int16 itemnum,RectangleType *bounds,
		 Char **itemtext)
{
    char amountstr[15];
    Int16 drawx,drawy;
    FontID oldfont=0; /* make gcc happy */
    Int16 mycuridx;

    mycuridx = filterCuridx == -1 ? 0 : filterCuridx;
    
    WinDrawChars(itemtext[itemnum],StrLen(itemtext[itemnum]),
		 bounds->topLeft.x,bounds->topLeft.y);

    drawy = bounds->topLeft.y;

    float_to_str(amountstr,reportList[itemnum].costs,2);
    drawx = bounds->topLeft.x + bounds->extent.x -
	60 - FntCharsWidth(amountstr,StrLen(amountstr));
    WinDrawChars(amountstr,StrLen(amountstr),drawx,drawy);

    if ((reportList[itemnum].balance > 0 && !activeBalance) 
	|| (reportList[itemnum].balance < 0 && activeBalance))
	    oldfont = FntSetFont(boldFont);

    float_to_str(amountstr,reportList[itemnum].balance,2);
    StrCat(amountstr,currencyList->list[mycuridx]->name);
    drawx = bounds->topLeft.x + bounds->extent.x - 
	FntCharsWidth(amountstr,StrLen(amountstr));
    WinDrawChars(amountstr,StrLen(amountstr),drawx,drawy);

    if ((reportList[itemnum].balance > 0 && !activeBalance) 
	|| (reportList[itemnum].balance < 0 && activeBalance))
	    FntSetFont(oldfont);

}

/***********************************************************************
 *
 * FUNCTION:     people_fill_reportlist
 * 
 * DESCRIPTION:  Fill the reportlist with costs and balance for each
 *               person
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
people_fill_reportlist(void)
{
    Int16 i,j,totalcount;
    itemType *item;
    Int16 mycuridx;

    mycuridx = filterCuridx == -1 ? 0 : filterCuridx;

    for (i=0;i<peopleList->nitems;i++) 
	reportList[i].costs = reportList[i].balance = 0.0;

    totalcount = db_num_items();
    for (i=0;i<totalcount;i++) {
	item = db_get_item(i);
	for (j=0;j<peopleList->nitems;j++) {
	    reportList[j].costs += get_item_costs(item,
						  peopleList->people[j].id,
						  USERCOST,mycuridx);
	    reportList[j].balance += get_item_costs(item,
						    peopleList->people[j].id,
						    USERBALANCE,mycuridx);

	}
	db_free_item(item);
    }
}

/***********************************************************************
 *
 * FUNCTION:     people_init
 * 
 * DESCRIPTION:  Constructor for formPeople
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
people_init(void)
{
    peopleList = db_get_peoplelist();
    reportList = MemPtrNew(sizeof(*reportList) * peopleList->nitems + 1);
    people_fill_reportlist();
    LstSetListChoices(GetObjectPtr(lstPeople),
		      peopleList->names,peopleList->nitems);
    LstSetDrawFunction(GetObjectPtr(lstPeople),people_draw_item);
}

/***********************************************************************
 *
 * FUNCTION:     people_destroy
 * 
 * DESCRIPTION:  Destructor for formPeople
 *
 * PARAMETERS:   Nothing
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/
static void
people_destroy(void)
{
    db_free_peoplelist(peopleList);
    MemPtrFree(reportList);
}

/***********************************************************************
 *
 * FUNCTION:     PeopleFormHandleEvent
 * 
 * DESCRIPTION:  Event handler for formPeople
 *
 * PARAMETERS:   event
 *
 * RETURN:       handled
 *      
 ***********************************************************************/
Boolean 
PeopleFormHandleEvent (EventPtr e)
{
    Boolean handled = false;
    FormPtr frm;
    ListPtr list;
    Int16 i;
    
    switch (e->eType) {
    case frmOpenEvent:
	frm = FrmGetActiveForm();
	people_init();
	FrmDrawForm(frm);
	handled = true;
	break;
	
    case frmCloseEvent:
	people_destroy();
	handled = false;
	break;

    case keyDownEvent:
	list = GetObjectPtr(lstPeople);
	frm = FrmGetActiveForm();
	switch (e->data.keyDown.chr) {
	case pageDownChr:
	    FrmHideObject(frm,FrmGetObjectIndex(frm,lstPeople));
	    LstScrollList(list,winDown,LstGetVisibleItems(list)-1);
	    FrmShowObject(frm,FrmGetObjectIndex(frm,lstPeople));	    
	    handled = true;
	    break;
	case pageUpChr:
	    FrmHideObject(frm,FrmGetObjectIndex(frm,lstPeople));
	    LstScrollList(list,winUp,LstGetVisibleItems(list)-1);
	    FrmShowObject(frm,FrmGetObjectIndex(frm,lstPeople));	    
	    handled = true;
	    break;
	}
	break;

    case ctlSelectEvent:
	switch(e->data.ctlSelect.controlID) {
	case btnPeopleNew:
	    if (person_change(-1)) 
		FrmUpdateForm(frmPeople,updPeopleList);
	    handled = true;
	    break;
	case btnPeopleModify:
	    i = LstGetSelection(GetObjectPtr(lstPeople));
	    if (i != noListSelection) {
		if (person_change(i)) 
		    FrmUpdateForm(frmPeople,updPeopleList);
	    }
	    handled = true;
	    break;
	case btnPeopleDone:
	    people_destroy();
	    FrmReturnToForm(0);
	    FrmUpdateForm(frmMain,updPeopleList);
	    handled = true;
	    break;
	case btnPeopleDelete:
	    i = LstGetSelection(GetObjectPtr(lstPeople));
	    if (i != noListSelection) {
		dbItemList *itemList;
		itemList = db_get_itemlist(peopleList->people[i].id);
		if (itemList->nitems != 0) 
		    FrmAlert(altUserHasItem);
		else if (FrmCustomAlert(altDeleteUserOK,
					peopleList->names[i],
					NULL,NULL) == 0) {
		    db_delete_person(peopleList->people[i].id);
		    FrmUpdateForm(frmPeople,updPeopleList);
		}
		db_free_itemlist(itemList);
	    }
	    handled = true;
	    break;
	}
	break;
    case frmUpdateEvent:
	if (e->data.frmUpdate.updateCode == updPeopleList) {
	    people_destroy();
	    people_init();
	    LstDrawList(GetObjectPtr(lstPeople));
	    handled = true;
	}
    default:
        break;
    }

    return handled;
}

